package main

import (
	"flag"
	"fmt"
	"strings"

	"git.sr.ht/~tslocum/desktop"
	"git.sr.ht/~tslocum/gmenu/pkg/gmenu"
	"github.com/kballard/go-shellquote"
)

func updateEntries(input string) {
	gmenu.FilterEntries()

	optionsList.selected = 0
	optionsList.origin = 0
	defer updateEntryInfo()

	optionsList.options = nil
	for _, entry := range gmenu.FilteredEntries {
		optionsList.options = append(optionsList.options, entry.Label)
	}
	if input != "" {
		optionsList.options = append(optionsList.options, input)
	}
}

func selectedIndex() int {
	if optionsList == nil {
		return -1
	}

	return optionsList.selected
}

func selectedEntry() *desktop.Entry {
	i := selectedIndex()
	if len(gmenu.FilteredEntries) == 0 || i < 0 || i > len(gmenu.FilteredEntries)-1 {
		return nil
	}

	return gmenu.FilteredEntries[i].Entry
}

func updateEntryInfo() {
	if config.HideAppDetails {
		return
	}

	var exLine, comLine string
	entry := selectedEntry()
	if entry != nil {
		if entry.Type == desktop.Application {
			exLine = entry.Exec
		} else { // Type == desktop.Link
			exLine = config.BrowserCommand() + " " + entry.URL
		}

		comLine = entry.Comment
	} else {
		exLine = fmt.Sprintf("bash -c %s", strings.TrimSpace(inputView.GetText()))

		comLine = "Shell command"
	}

	appDetailsView = appDetailsView.SetText(exLine + "\n\n" + comLine)
}

func listSelect() error {
	defer closeGUI()

	var (
		execute           string
		runInTerminal     bool
		waitUntilFinished bool
	)
	entry := selectedEntry()
	if entry == nil {
		waitUntilFinished = true
		execute = inputView.GetText()
	} else if entry.Type == desktop.Application {
		if entry.Terminal {
			runInTerminal = true
		}

		execute = entry.ExpandExec(shellquote.Join(flag.Args()...))
	} else { // Type == desktop.Link
		execute = shellquote.Join(config.BrowserCommand(), entry.URL)
	}

	closeGUI()

	path := ""
	if entry != nil {
		path = entry.Path
	}

	return gmenu.Run(&config.Config, execute, path, runInTerminal, waitUntilFinished)
}
