package main

import (
	"flag"
	"log"
	"os"

	"git.sr.ht/~tslocum/gmenu/pkg/gmenu"
	"github.com/mattn/go-isatty"
)

type Config struct {
	gmenu.Config

	EnableMouse bool
}

var (
	config = &Config{}

	done = make(chan bool)
)

func init() {
	gmenu.SharedInit(&config.Config)

	flag.BoolVar(&config.EnableMouse, "mouse", false, "enable mouse support")
}

func main() {
	flag.Parse()

	tty := isatty.IsTerminal(os.Stdout.Fd()) || isatty.IsCygwinTerminal(os.Stdout.Fd())
	if !tty {
		log.Fatal("failed to start gmenu: non-interactive terminals are not supported")
	}

	gmenu.LoadEntries(&config.Config)

	app, err := initGUI()
	if err != nil {
		panic(err)
	}

	go func() {
		if err := app.Run(); err != nil {
			panic(err)
		}

		done <- true
	}()

	<-done

	closeGUI()
}
