package main

import (
	"crypto/md5"
	"fmt"
	"log"
	"os"
	"path"

	"github.com/gotk3/gotk3/gdk"
	"github.com/gotk3/gotk3/gtk"
)

const CachedIconCompressionLevel = 3

var (
	iconTheme    *gtk.IconTheme
	iconCacheDir string
)

func loadIcon(icon string) (*gdk.Pixbuf, error) {
	cachedIcon := path.Join(iconCacheDir, fmt.Sprintf("%x.png", md5.Sum([]byte(icon))))

	if _, err := os.Stat(cachedIcon); !os.IsNotExist(err) {
		return gdk.PixbufNewFromFileAtSize(cachedIcon, iconSize, iconSize)
	}

	var (
		pbuf *gdk.Pixbuf
		err  error
	)
	if path.IsAbs(icon) {
		pbuf, err = gdk.PixbufNewFromFileAtSize(icon, iconSize, iconSize)
	} else {
		pbuf, err = iconTheme.LoadIcon(icon, iconSize, gtk.ICON_LOOKUP_USE_BUILTIN)
	}
	if err != nil {
		return nil, err
	}

	if pbuf.GetWidth() != iconSize || pbuf.GetHeight() != iconSize {
		pbuf, _ = pbuf.ScaleSimple(iconSize, iconSize, gdk.INTERP_BILINEAR)
	}

	go cacheIcon(cachedIcon, pbuf)

	return pbuf, nil
}

func cacheIcon(cachedIcon string, pbuf *gdk.Pixbuf) {
	f, err := os.OpenFile(cachedIcon, os.O_CREATE|os.O_RDWR, 0644)
	if err != nil {
		log.Println(err)
		return
	}

	pbuf.SavePNG(f.Name(), CachedIconCompressionLevel)
}
