package main

import (
	"flag"
	"log"
	"os"
	"path"
	"time"

	"git.sr.ht/~tslocum/gmenu/pkg/gmenu"
	"github.com/gotk3/gotk3/gdk"
	"github.com/gotk3/gotk3/glib"
	"github.com/gotk3/gotk3/gtk"
)

const (
	appID = "space.rocketnine.gmenu"
)

type Config struct {
	gmenu.Config

	Width, Height int
	Resizable     bool
	Fullscreen    bool

	HideAppIcons bool
}

var (
	config    = &Config{}
	listBox   *gtk.ListBox
	inputView *gtk.TextView

	t      = time.Now()
	loaded = make(chan bool)
)

func init() {
	gmenu.SharedInit(&config.Config)

	flag.IntVar(&config.Width, "width", 600, "window width")
	flag.IntVar(&config.Height, "height", 200, "window height")
	flag.BoolVar(&config.Resizable, "resizable", false, "allow window to be resized")
	flag.BoolVar(&config.Fullscreen, "fullscreen", false, "start fullscreen")
	flag.BoolVar(&config.HideAppIcons, "no-icons", false, "hide application icons")
}

func load() {
	flag.Parse()

	homeDir, err := os.UserHomeDir()
	if err != nil {
		log.Fatal(err)
	}

	iconCacheDir = path.Join(homeDir, ".cache", "gmenu", "icons")
	if _, err := os.Stat(iconCacheDir); os.IsNotExist(err) {
		os.MkdirAll(iconCacheDir, 0744) // TODO: Warn and disable cache on error
	}

	gmenu.LoadEntries(&config.Config)

	for i, l := range gmenu.Names {
		gmenu.FilteredEntries = append(gmenu.FilteredEntries, &gmenu.ListEntry{Label: l, Entry: gmenu.Entries[i]})
	}

	gmenu.Entries = append(gmenu.Entries, nil)
	gmenu.Names = append(gmenu.Names, "")
	gmenu.FilteredEntries = append(gmenu.FilteredEntries, &gmenu.ListEntry{Label: "", Entry: nil})

	loaded <- true
}

func main() {
	go load()

	gtk.Init(nil)

	w, err := gtk.WindowNew(gtk.WINDOW_TOPLEVEL)
	if err != nil {
		log.Fatal("failed to create application window:", err)
	}

	w.SetTitle("gmenu")

	w.SetDecorated(false)
	w.SetBorderWidth(0)
	w.Stick()
	w.SetKeepAbove(true)
	w.SetTypeHint(gdk.WINDOW_TYPE_HINT_UTILITY)

	if !config.Fullscreen {
		w.SetResizable(config.Resizable)
		w.SetSizeRequest(config.Width, config.Height)
		w.SetPosition(gtk.WIN_POS_CENTER)
	} else {
		w.Fullscreen()
	}

	_, err = w.Connect("destroy", func() {
		os.Exit(0)
	})
	if err != nil {
		log.Fatal("failed to create application window:", err)
	}

	setupKeyBindings(w)

	go gmenu.HandleInput(func(input string) {
		_, err := glib.IdleAdd(updateList, input)
		if err != nil {
			log.Fatal(err)
		}
	})

	<-loaded

	container := newBox(gtk.ORIENTATION_VERTICAL)

	initList(container)

	w.Add(container)

	w.ShowAll()

	gtk.Main()
}
